;==========================================================================
; resh2inc.exe reads a resource.h file and creates a resource.inc file.
; This permits you to use a header file in your resource definition and
; also use those same #defines as equates in your MASM32 program.
; simply modify the MASM32 bat files to run this program before running the
; assembles.  This program does the following:
; 1) removes the # from all elif, ifdef, if, else, ifndef, endif, and elif
; is replaced by elseif
; 2) replaces all C/C++ comments (multi and single line) with valid 
; assembler comment lines.
; 3) Should handle most equate expressions
; 4) copies strings to equate strings
; 5) changes #defines to equates
; 6) fixes the 0x (replaces with h suffix and adds leading 0 if needed)
; 7) The UL, and L suffixes are removed.
;
; Send suggestions or comments to Dan
; danesmith@comcast.net
;==========================================================================
      .386
      .model flat, stdcall  ; 32 bit memory model
      option casemap :none  ; case sensitive
;      IS_DEBUG EQU 1
;--------------------------------------------------------------------------
;     include files
;--------------------------------------------------------------------------
      include \MASM32\INCLUDE\windows.inc
      include \MASM32\INCLUDE\masm32.inc
      include \MASM32\INCLUDE\gdi32.inc
      include \MASM32\INCLUDE\user32.inc
      include \MASM32\INCLUDE\kernel32.inc
      include \MASM32\INCLUDE\Comctl32.inc
      include \MASM32\INCLUDE\comdlg32.inc
      include \MASM32\INCLUDE\oleaut32.inc
;      include \MASM32\PROJECTS\INCLUDE\mydebug.inc
;--------------------------------------------------------------------------
;     libraries
;--------------------------------------------------------------------------
      includelib \MASM32\LIB\masm32.lib
      includelib \MASM32\LIB\gdi32.lib
      includelib \MASM32\LIB\user32.lib
      includelib \MASM32\LIB\kernel32.lib
      includelib \MASM32\LIB\Comctl32.lib
      includelib \MASM32\LIB\comdlg32.lib
      includelib \MASM32\LIB\oleaut32.lib
;--------------------------------------------------------------------------
      ReadHeader    PROTO 
      FormatInclude PROTO
      IsDigit       PROTO
      IsSpace       PROTO
      IsPound       PROTO
      IsOperator    PROTO
      skip2eol      PROTO
      skipmulti     PROTO
      skipstr       PROTO
      CmpNames      PROTO, :DWORD, :DWORD, :DWORD, :DWORD
      WriteInclude  PROTO

.data
    len         dd 0
    InMem       dd 0
    OutMem      dd 0
    ctr         dd 0
    Equate      db 'equ '
    inResource  db 'resource.h',0
    outResource db 'resource.inc',0
    numbers     db '0123456789',0
    digit       db 0,0
    operators   db '<>()+-*/|&',0
    spaces      db ' ',9,13,10,0
    pdefine     db 'define'
    pelif       db 'elif'
    pifdef      db 'ifdef'
    pif         db 'if'
    pelse       db 'else'
    pifndef     db 'ifndef'
    pendif      db 'endif' 
    pelseif     db 'elseif'
    digiflag    dd 0
    operflag    dd 0
    spacflag    dd 0
    poundflag   dd 0
;_dbg
.code
;_fun
;======================================================================

start:
;_trc
    invoke ReadHeader
    .if eax > 0
        invoke FormatInclude
        invoke WriteInclude
    .endif
    mov eax,0
    invoke ExitProcess, eax

;==========================================================================

ReadHeader proc
    LOCAL hFile :DWORD
    LOCAL br    :DWORD

    invoke CreateFile,addr inResource,GENERIC_READ,FILE_SHARE_READ,
            NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL
    mov hFile, eax                      ; Open input file and get handle
    .if eax == 0ffffffffh
        mov eax,0
        ret
    .endif
    

    invoke GetFileSize,hFile,NULL       ; get its size in bytes
    mov len, eax

    invoke SysAllocStringByteLen,0,len  ; get memory for it
    mov InMem, eax

    invoke ReadFile,hFile,InMem,len,ADDR br,NULL
    invoke CloseHandle,hFile            ; read the file and clean up

    mov eax,1
    ret
ReadHeader endp

;==========================================================================
FormatInclude proc
    LOCAL state:DWORD
    LOCAL hexswi:DWORD
    LOCAL cmpswi:DWORD
    LOCAL pname:DWORD

    mov eax,len                ; get one more
    inc eax                    ; byte for null char
    invoke SysAllocStringByteLen,0,eax
    mov OutMem, eax            ; get some memory in which to put the output

    ; pick a few STATE values for this engine
    MOVECHAR  equ 1            ; simply move characters
    MOVESPACE equ 2            ; move spaces between #define and name
    MOVENAME  equ 3            ; move the name 
    FINDVAL   equ 4            ; skip spaces to find the value
    FIXHEX    equ 5            ; find end of hex value and add h
    FIXNUM    equ 6            ; find end of a number and remove any UL or L chars
    EXPRESS   equ 7            ; move name within an expression

    mov state,MOVECHAR         ; start out in the move char state
    mov eax,InMem              ; set up our pointers
    mov ebx,OutMem
    mov edx,0                  ; clear work register
    mov ecx,0                  ; clear count register
    mov ctr,ecx
    .while ecx < len                       ; and begin to loop
        mov dl,[eax]                       ; get the first character
        mov dh,0                           ; clear look ahead character
        mov ecx,ctr
        inc ecx                            ; count the character
        mov ctr,ecx
        inc eax                            ; and skip it
        .if state == MOVECHAR || state == EXPRESS   ; is this the move character state?
            .if dl == '\' && state == EXPRESS ; is this a continuation char in an expression
                invoke skip2eol
                .continue
            .endif
            .if dl == '/'                  ; is this a possible comment?
                mov dh,[eax]               ; then peek at next char
                .if dh == '/'              ; is this a single line C++ comment
                    inc eax                ;
                    mov ecx,ctr
                    inc ecx
                    mov ctr,ecx
                    mov dl,';'             ; then make it an ASM comment
                    mov state,MOVECHAR     ; will end expression state if comment
                    mov [ebx],dl           ; write the new char
                    inc ebx                ; and skip it
                    invoke skip2eol        ;
                    .continue              ; before going on
                .endif
                .if dh == '*'              ; is this a multi line C comment?  
                    mov state,MOVECHAR     ; will end expression state if comment
                    invoke skipmulti       ; skip the entire comment
                    .continue              ; go get next char 
                .endif
            .endif
            .if dl == '#'                    ; is this a possible define?
                mov pname,eax
                push eax
                invoke CmpNames,pname,addr pdefine,6,addr cmpswi
                pop eax
                .if cmpswi == 1
                    add eax,6                ; skip #define
                    mov ecx,ctr
                    add ecx,6                ; count it
                    mov ctr,ecx
                    mov state,MOVESPACE      ; if so and we have whitespace
                    .continue                ; then look for next char
                .endif  
                push eax
                invoke CmpNames,pname,addr pelif,4,addr cmpswi
                pop eax
                .if cmpswi == 1
                    invoke MemCopy,ebx,addr pelseif,6
                    add eax,4                ; skip #elif
                    mov ecx,ctr
                    add ecx,4
                    mov ctr,ecx
                    add ebx,4
                    .continue
                .endif  
                mov dl,' '
                push eax
                invoke CmpNames,pname,addr pifdef,5,addr cmpswi
                pop eax
                mov ecx,ctr
                .if cmpswi == 1
                    mov [ebx],dl               ; overlay # with a ; or space
                    .continue
                .endif  
                push eax
                invoke CmpNames,pname,addr pelse,4,addr cmpswi
                pop eax
                mov ecx,ctr
                .if cmpswi == 1
                    mov [ebx],dl               ; overlay # with a ; or space
                    .continue
                .endif  
                push eax
                invoke CmpNames,pname,addr pifndef,5,addr cmpswi
                pop eax
                mov ecx,ctr
                .if cmpswi == 1
                    mov [ebx],dl               ; overlay # with a space
                    .continue
                .endif  
                push eax
                invoke CmpNames,pname,addr pif,2,addr cmpswi
                pop eax
                mov ecx,ctr
                .if cmpswi == 1
                    mov [ebx],dl               ; overlay # with a space
                    .continue
                .endif  
                push eax
                invoke CmpNames,pname,addr pendif,5,addr cmpswi
                pop eax
                mov ecx,ctr
                .if cmpswi == 1
                     mov [ebx],dl               ; overlay # with a space
                    .continue
                .endif  
                mov dl,';'
                mov [ebx],dl               ; overlay # with a ';'
            .endif
            mov [ebx],dl                   ; move all other chars 
            inc ebx                        ; prepare for next char                                     
            .if state == EXPRESS           ; is this the expression state?
                .if dl == 13               ; end of line
                    mov state,MOVECHAR     ; will end expression state if comment
                    .continue
                .endif
                invoke IsOperator          ; check for an operator
                .if operflag > 0           ; if we find an operator         
                    mov state,FINDVAL      ; go look for numbers to fix
                .endif                     
            .endif
            .continue                      ; and get next char
        .endif
        .if state == MOVESPACE             ; we are moving spaces between
            mov [ebx],dl                   ; #defines and their names
            inc ebx                        ; so move the char and prepare for next
            invoke IsSpace
            .if spacflag < 1
               mov state,MOVENAME          ; we found the name of the equate
            .endif                         ; mark the name state and 
            .continue                      ; go move it's chars
        .endif
        .if state == MOVENAME              ; here we are moving the equate name
            mov [ebx],dl
            inc ebx
            invoke IsSpace
            .if spacflag > 0
                push ebp
                lea ebp,Equate             ; when white space is found move 'equ '
                mov ebp,[ebp]
                mov [ebx],ebp
                add ebx,3                  ; don't count space from equ
                pop ebp
                mov [ebx],dl               ; but move the whitespace we found
                inc ebx
                mov state,FINDVAL          ; now, find the value
            .endif
        .endif
        .if state == FINDVAL              ; we need to fix some numbers
            invoke IsSpace
            .if spacflag > 0
               .if dl == 13
                   mov state,MOVECHAR
               .endif
               mov [ebx],dl               ; But we have to find them first
               inc ebx                    ; so just move blank spaces
               .continue                  ; and go on
            .endif
            .if dl == '"' || dl == "'"
                invoke skipstr
                mov state,MOVECHAR
                .continue
            .endif
            invoke IsDigit                ; check if numeric
            .if digiflag < 1 
                mov [ebx],dl              ; definitely not numeric
                inc ebx                   ; just move it and 
                mov state,EXPRESS         ; treat rest of it as expression    
                .continue                 ; and loop
            .endif
            mov state,FIXNUM              ; prepare for a number
            .if dl == '0'                 ; check for hex number
                mov dh,[eax]              ; get next digit      
                .if dh == 'x'             ; is this a hex number?
                    inc eax               ; skip hex notation
                    mov dl,[eax]          ; get the next char
                    inc eax
                    call IsDigit
                    .if digiflag < 1                             
                        mov byte ptr[ebx],'0' ; prefix hex alpha char with zero
                        inc ebx           ; and skip zero
                    .endif
                    mov state,FIXHEX      ; we found hex, need to append h
                .endif
            .endif                          
            mov [ebx],dl                   ; now move the character
            inc ebx                        ; skip it
            .continue                      ; and go around
        .endif
        .if state == FIXHEX              ; we need to append the 'h'
            invoke IsSpace
            invoke IsOperator       ; check for an operator
            .if spacflag > 0 || operflag > 0
                mov dh,[ebx-1]
                .if dh == 'L'             ; did it end in L
                   dec ebx
                   mov dh,[ebx-1]
                   .if dh == 'U'          ; or UL
                       dec ebx            ; fix it
                   .endif 
                .endif
                mov byte ptr [ebx],'h'    ; blank found append 'h'
                inc ebx                   ; and skip it                 
               .if dl == 13
                   mov state,MOVECHAR     ; end of line, no value
               .else    
                   mov state,EXPRESS      ; now, go do expression
               .endif
            .endif
            mov [ebx],dl                  ; move in our space or char               
            inc ebx                       ; and skip it
            .continue
        .endif
        .if state == FIXNUM              ; we need to remove any 'L' or UL
            invoke IsSpace
            invoke IsOperator       ; check for an operator
            .if spacflag > 0 || operflag > 0
                mov dh,[ebx-1]
               .if dl == 13
                   mov state,MOVECHAR     ; end of line, no value
               .else
                   mov state,EXPRESS      ; now, go back to moving all chars
               .endif
                .if dh == 'L'             ; did it end in L
                   mov dh,[ebx-2]
                   .if dh == 'U'          ; or UL
                       dec ebx            ; fix it
                   .endif 
                   mov byte ptr [ebx-1],dl  ; clear the L (already counted)
                   .continue               ; now go on
                .endif
            .endif
            mov [ebx],dl                  ; now move in our space or char               
            inc ebx                       ; and skip it
        .endif
    .endw
    mov dl,0                              ; null terminate the output
    mov [ebx],dl                           
    invoke lnstr,OutMem                   ; get the new length
    mov len,eax
    invoke SysFreeString,InMem            ; free input memory and return
    ret
FormatInclude endp

;==========================================================================

IsDigit proc
    push eax              ; save two regs 
    push edx 
    mov digit,dl          ; make the digit a string
    invoke InString,1,addr numbers,addr digit  
    mov digiflag,eax       ; save type of data
    pop edx
    pop eax               ; restore regs
    ret
IsDigit endp

;==========================================================================

IsOperator proc
    push eax              ; save two regs 
    push edx 
    mov digit,dl          ; make the digit a string
    invoke InString,1,addr operators,addr digit  
    mov operflag,eax       ; save type of data
    pop edx
    pop eax               ; restore regs
    ret
IsOperator endp

;==========================================================================

IsSpace proc
    push eax              ; save two regs 
    push edx 
    mov digit,dl          ; make the digit a string
    invoke InString,1,addr spaces,addr digit  
    mov spacflag,eax       ; save type of data
    pop edx
    pop eax               ; restore regs
    ret
IsSpace endp

;==========================================================================

skip2eol proc
    mov [ebx],dl                  ; move the \
    inc ebx                       ; and skip it
    mov ecx,ctr
    .while ecx < len              ; now search to find the end crlf
        mov dl,[eax]              ; get the char
        inc eax                   ; and skip it
        mov ecx,ctr
        inc ecx                   ; count it too
        mov ctr,ecx
        mov [ebx],dl
        inc ebx
        .if dl == 13              ; end of line
            .break                ; start processing at line feed
        .endif    
    .endw
    ret
skip2eol endp

;==========================================================================

skipstr proc
    LOCAL prev:BYTE
    mov byte ptr [ebx],'<'        ; move the asm quote
    inc ebx                       ; and skip it
    mov dh,dl                     ; save the quote
    mov ecx,ctr
    .while ecx < len                 ; now search to find the end quote
        mov prev,dl 
        mov dl,[eax]                 ; get the char
        inc eax                      ; and skip it
        mov ecx,ctr
        inc ecx                      ; count it too
        mov ctr,ecx
        mov [ebx],dl                  ; move the char
        inc ebx                       ; and skip it
        .if dl == dh && prev != '\'  ; end of quote and not escaped
            mov byte ptr [ebx-1],'>' ; fix the quote
            .break                   ; start processing after it
        .endif    
    .endw
    ret
skipstr endp

;==========================================================================

skipmulti proc
    inc eax                ; skip the * character after the /
    mov byte ptr [ebx],';'
    inc ebx
    mov ecx,ctr
    inc ecx                ; and count it
    .while ecx < len       ; now search to find the end */
        mov dl,[eax]         ; get the char
        inc eax              ; and skip it
        mov ecx,ctr
        inc ecx              ; count it too
        mov ctr,ecx
        .if dl == '*'        ; is it a possible end?
           mov dh,[eax]        ; then peek at next char
           .if dh == '/'       ; is it the end?    
              inc eax            ; then skip
              mov ecx,ctr
              inc ecx            ; and count it
              mov ctr,ecx
              .break             ; and continue outer loop
           .endif
        .else
            mov [ebx],dl
            inc ebx
            .if dl == 10
                mov byte ptr [ebx],';'
                inc ebx
            .endif
        .endif
    .endw
    ret
skipmulti endp

;==========================================================================

CmpNames proc left:DWORD, right:DWORD, mlen:DWORD, mret:DWORD
    pushad
    mov eax,left
    mov ebx,right
    mov ecx,mlen
    .repeat
        mov dl,[eax]
        cmp dl,[ebx]
        jne nomatch
        inc eax
        inc ebx
    .untilcxz
    mov eax,1
    jmp cmpdone
nomatch:
    mov eax,0
cmpdone:
    mov ecx,mret
    mov [ecx],eax
    popad
    ret
CmpNames endp

;==========================================================================

WriteInclude proc
    LOCAL hFile :DWORD
    LOCAL bw    :DWORD

    invoke CreateFile,addr outResource, ; pointer to name of the file
            GENERIC_WRITE,              ; access (read-write) mode
            NULL,                       ; share mode
            NULL,                       ; pointer to security attributes
            CREATE_ALWAYS,              ; how to create
            FILE_ATTRIBUTE_NORMAL,      ; file attributes
            NULL
    mov hFile,eax
    

    invoke WriteFile,hFile,OutMem,len,ADDR bw,NULL

    invoke SysFreeString,OutMem          ; free memory after the write
    invoke CloseHandle,hFile             ; clean up the file

    ret                                  ; and return

WriteInclude endp

end start
